@@ -7,12 +7,13 @@
 #include <RcppArmadillo.h>
 #include <iostream>
 #include <vector>
-#include <cassert>
 #include "basis.h"
+#include <boost/math/special_functions.hpp>
 
 using namespace Rcpp;
 using namespace arma;
 //[[Rcpp::depends(RcppArmadillo)]]
+//[[Rcpp::depends(BH)]]
 
 
 //' Isotropic exponential covariance function
@@ -35,10 +36,10 @@ using namespace arma;
 //' The nugget value \eqn{ \sigma^2 \tau^2 } is added to the diagonal of the covariance matrix.
 //' NOTE: the nugget is \eqn{ \sigma^2 \tau^2 }, not \eqn{ \tau^2 }. 
 // [[Rcpp::export]]
-arma::mat exponential_isotropic(NumericVector covparms, NumericMatrix locs ){
+arma::mat exponential_isotropic(arma::vec covparms, arma::mat locs ){
 
-    int dim = locs.ncol();
-    int n = locs.nrow();
+    int dim = locs.n_cols;
+    int n = locs.n_rows;
     double nugget = covparms( 0 )*covparms( 2 );
     // create scaled locations
     mat locs_scaled(n,dim);
@@ -55,7 +56,7 @@ arma::mat exponential_isotropic(NumericVector covparms, NumericMatrix locs ){
         for(int j=0; j<dim; j++){
             d += pow( locs_scaled(i1,j) - locs_scaled(i2,j), 2.0 );
         }
-        d = pow( d, 0.5 );
+        d = std::sqrt( d );
         if( d == 0.0 ){
             covmat(i2,i1) = covparms(0);
         } else {
@@ -73,10 +74,10 @@ arma::mat exponential_isotropic(NumericVector covparms, NumericMatrix locs ){
 
 //' @describeIn exponential_isotropic Derivatives of isotropic exponential covariance
 // [[Rcpp::export]]
-arma::cube d_exponential_isotropic(NumericVector covparms, NumericMatrix locs ){
+arma::cube d_exponential_isotropic(arma::vec covparms, arma::mat locs ){
 
-    int dim = locs.ncol();
-    int n = locs.nrow();
+    int dim = locs.n_cols;
+    int n = locs.n_rows;
     //double nugget = covparms( 0 )*covparms( 2 );
     // create scaled locations
     mat locs_scaled(n,dim);
@@ -86,13 +87,13 @@ arma::cube d_exponential_isotropic(NumericVector covparms, NumericMatrix locs ){
         }
     }
     // calculate derivatives
-    arma::cube dcovmat = arma::cube(n,n,covparms.length(), fill::zeros);
+    arma::cube dcovmat = arma::cube(n,n,covparms.n_elem, fill::zeros);
     for(int i1=0; i1<n; i1++){ for(int i2=0; i2<=i1; i2++){
         double d = 0.0;
         for(int j=0; j<dim; j++){
             d += pow( locs_scaled(i1,j) - locs_scaled(i2,j), 2.0 );
         }
-        d = pow( d, 0.5 );
+        d = std::sqrt( d );
         
         dcovmat(i1,i2,0) += std::exp(-d);
         dcovmat(i1,i2,1) += covparms(0)*std::exp(-d)*d/covparms(1);
@@ -100,7 +101,7 @@ arma::cube d_exponential_isotropic(NumericVector covparms, NumericMatrix locs ){
             dcovmat(i1,i2,0) += covparms(2);
             dcovmat(i1,i2,2) += covparms(0); 
         } else { // fill in opposite entry
-            for(int j=0; j<covparms.length(); j++){
+            for(int j=0; j<covparms.n_elem; j++){
                 dcovmat(i2,i1,j) = dcovmat(i1,i2,j);
             }
         }
@@ -111,7 +112,6 @@ arma::cube d_exponential_isotropic(NumericVector covparms, NumericMatrix locs ){
 
 
 
-
 //' Isotropic Matern covariance function
 //'
 //' From a matrix of locations and covariance parameters of the form
@@ -132,12 +132,12 @@ arma::cube d_exponential_isotropic(NumericVector covparms, NumericMatrix locs ){
 //' The nugget value \eqn{ \sigma^2 \tau^2 } is added to the diagonal of the covariance matrix.
 //' NOTE: the nugget is \eqn{ \sigma^2 \tau^2 }, not \eqn{ \tau^2 }. 
 // [[Rcpp::export]]
-arma::mat matern_isotropic(NumericVector covparms, NumericMatrix locs ){
+arma::mat matern_isotropic(arma::vec covparms, arma::mat locs ){
 
-    int dim = locs.ncol();
-    int n = locs.nrow();
+    int dim = locs.n_cols;
+    int n = locs.n_rows;
     double nugget = covparms( 0 )*covparms( 3 );
-    double normcon = covparms(0)/(pow(2.0,covparms(2)-1.0)*Rf_gammafn(covparms(2)));
+    double normcon = covparms(0)/(pow(2.0,covparms(2)-1.0)*boost::math::tgamma(covparms(2) ));
     
     // create scaled locations
     mat locs_scaled(n,dim);
@@ -164,7 +164,7 @@ arma::mat matern_isotropic(NumericVector covparms, NumericMatrix locs ){
             } else {
                 // calculate covariance            
                 covmat(i2,i1) = normcon*
-                    pow( d, covparms(2) )*Rf_bessel_k(d,covparms(2),1.0);
+                    pow( d, covparms(2) )*boost::math::cyl_bessel_k(covparms(2), d);
             }
             // add nugget
             if( i1 == i2 ){ covmat(i2,i2) += nugget; } 
@@ -179,15 +179,15 @@ arma::mat matern_isotropic(NumericVector covparms, NumericMatrix locs ){
 
 //' @describeIn matern_isotropic Derivatives of isotropic Matern covariance
 // [[Rcpp::export]]
-arma::cube d_matern_isotropic(NumericVector covparms, NumericMatrix locs ){
+arma::cube d_matern_isotropic(arma::vec covparms, arma::mat locs ){
 
-    int dim = locs.ncol();
-    int n = locs.nrow();
+    int dim = locs.n_cols;
+    int n = locs.n_rows;
     //double nugget = covparms( 0 )*covparms( 3 );
-    double normcon = covparms(0)/(pow(2.0,covparms(2)-1.0)*Rf_gammafn(covparms(2)));
+    double normcon = covparms(0)/(pow(2.0,covparms(2)-1.0)*boost::math::tgamma(covparms(2) ));
     double eps = 1e-8;
     double normconeps = 
-        covparms(0)/(pow(2.0,covparms(2)+eps-1.0)*Rf_gammafn(covparms(2)+eps));
+        covparms(0)/(pow(2.0,covparms(2)+eps-1.0)*boost::math::tgamma(covparms(2)+eps ));
     
     // create scaled locations
     mat locs_scaled(n,dim);
@@ -197,7 +197,7 @@ arma::cube d_matern_isotropic(NumericVector covparms, NumericMatrix locs ){
         }
     }
     // calculate derivatives
-    arma::cube dcovmat = arma::cube(n,n,covparms.length(), fill::zeros);
+    arma::cube dcovmat = arma::cube(n,n,covparms.n_elem, fill::zeros);
     for(int i1=0; i1<n; i1++){ for(int i2=0; i2<=i1; i2++){
         double d = 0.0;
         for(int j=0; j<dim; j++){
@@ -212,22 +212,22 @@ arma::cube d_matern_isotropic(NumericVector covparms, NumericMatrix locs ){
             dcovmat(i1,i2,1) += 0.0;
             dcovmat(i1,i2,2) += 0.0;
         } else {
-            cov = normcon*pow( d, covparms(2) )*Rf_bessel_k(d,covparms(2),1.0);
+            cov = normcon*pow( d, covparms(2) )*boost::math::cyl_bessel_k(covparms(2), d);
             // variance parameter
             dcovmat(i1,i2,0) += cov/covparms(0);
             // range parameter
             dcovmat(i1,i2,1) += normcon*pow(d,covparms(2))*
-                Rf_bessel_k(d,covparms(2)-1.0,1.0)*d/covparms(1);
+                boost::math::cyl_bessel_k(covparms(2)-1.0, d)*d/covparms(1);
             // smoothness parameter (finite differencing)
             dcovmat(i1,i2,2) += 
-                ( normconeps*pow(d,covparms(2)+eps)*Rf_bessel_k(d,covparms(2)+eps,1.0) -
+                ( normconeps*pow(d,covparms(2)+eps)*boost::math::cyl_bessel_k(covparms(2) + eps, d) -
                   cov )/eps;
         }
         if( i1 == i2 ){ // update diagonal entry
             dcovmat(i1,i2,0) += covparms(3);
             dcovmat(i1,i2,3) += covparms(0); 
         } else { // fill in opposite entry
-            for(int j=0; j<covparms.length(); j++){
+            for(int j=0; j<covparms.n_elem; j++){
                 dcovmat(i2,i1,j) = dcovmat(i1,i2,j);
             }
         }
@@ -262,10 +262,10 @@ arma::cube d_matern_isotropic(NumericVector covparms, NumericMatrix locs ){
 //' The nugget value \eqn{ \sigma^2 \tau^2 } is added to the diagonal of the covariance matrix.
 //' NOTE: the nugget is \eqn{ \sigma^2 \tau^2 }, not \eqn{ \tau^2 }. 
 // [[Rcpp::export]]
-arma::mat matern15_isotropic(NumericVector covparms, NumericMatrix locs ){
+arma::mat matern15_isotropic(arma::vec covparms, arma::mat locs ){
 
-    int dim = locs.ncol();
-    int n = locs.nrow();
+    int dim = locs.n_cols;
+    int n = locs.n_rows;
     double nugget = covparms( 0 )*covparms( 2 );
     // create scaled locations
     mat locs_scaled(n,dim);
@@ -301,10 +301,10 @@ arma::mat matern15_isotropic(NumericVector covparms, NumericMatrix locs ){
 //' @describeIn exponential_isotropic Derivatives of isotropic 
 //' matern covariance with smoothness 1.5
 // [[Rcpp::export]]
-arma::cube d_matern15_isotropic(NumericVector covparms, NumericMatrix locs ){
+arma::cube d_matern15_isotropic(arma::vec covparms, arma::mat locs ){
 
-    int dim = locs.ncol();
-    int n = locs.nrow();
+    int dim = locs.n_cols;
+    int n = locs.n_rows;
     //double nugget = covparms( 0 )*covparms( 2 );
     // create scaled locations
     mat locs_scaled(n,dim);
@@ -314,7 +314,7 @@ arma::cube d_matern15_isotropic(NumericVector covparms, NumericMatrix locs ){
         }
     }
     // calculate derivatives
-    arma::cube dcovmat = arma::cube(n,n,covparms.length(), fill::zeros);
+    arma::cube dcovmat = arma::cube(n,n,covparms.n_elem, fill::zeros);
     for(int i1=0; i1<n; i1++){ for(int i2=0; i2<=i1; i2++){
         double d = 0.0;
         for(int j=0; j<dim; j++){
@@ -328,7 +328,7 @@ arma::cube d_matern15_isotropic(NumericVector covparms, NumericMatrix locs ){
             dcovmat(i1,i2,0) += covparms(2);
             dcovmat(i1,i2,2) += covparms(0); 
         } else { // fill in opposite entry
-            for(int j=0; j<covparms.length(); j++){
+            for(int j=0; j<covparms.n_elem; j++){
                 dcovmat(i2,i1,j) = dcovmat(i1,i2,j);
             }
         }
