@@ -5,7 +5,6 @@
 #include <RcppArmadillo.h>
 #include <iostream>
 #include <vector>
-#include <cassert>
 #include "basis.h"
 #include "covmatrix_funs_01.h"
 
@@ -36,7 +35,7 @@ using namespace arma;
 //' The nugget value \eqn{ \sigma^2 \tau^2 } is added to the diagonal of the covariance matrix.
 //' NOTE: the nugget is \eqn{ \sigma^2 \tau^2 }, not \eqn{ \tau^2 }. 
 // [[Rcpp::export]]
-arma::mat matern_anisotropic2D(NumericVector covparms, NumericMatrix locs ){
+arma::mat matern_anisotropic2D(arma::vec covparms, arma::mat locs ){
     
     // covparms(0) = sigmasq
     // covparms(1) = L00
@@ -47,10 +46,10 @@ arma::mat matern_anisotropic2D(NumericVector covparms, NumericMatrix locs ){
     // nugget = sigmasq*tausq
     // overall variance = sigmasq*(1 + tausq) = sigmasq + nugget
     
-    //int dim = locs.ncol();
-    int n = locs.nrow();
+    //int dim = locs.n_cols;
+    int n = locs.n_rows;
     double nugget = covparms( 0 )*covparms( 5 );
-    double normcon = covparms(0)/(pow(2.0,covparms(4)-1.0)*Rf_gammafn(covparms(4)));
+    double normcon = covparms(0)/(pow(2.0,covparms(4)-1.0)*boost::math::tgamma(covparms(4) ));
     
     double b0 = covparms(1)*covparms(1);
     double b1 = covparms(2)*covparms(2)+covparms(3)*covparms(3);
@@ -72,7 +71,7 @@ arma::mat matern_anisotropic2D(NumericVector covparms, NumericMatrix locs ){
             } else {
                 // calculate covariance            
                 covmat(i2,i1) = normcon*
-                    pow( d, covparms(4) )*Rf_bessel_k(d,covparms(4),1.0);
+                    pow( d, covparms(4) )*boost::math::cyl_bessel_k(covparms(4), d);
             }
             // add nugget
             if( i1 == i2 ){ covmat(i2,i2) += nugget; } 
@@ -85,7 +84,7 @@ arma::mat matern_anisotropic2D(NumericVector covparms, NumericMatrix locs ){
 
 //' @describeIn matern_anisotropic2D Derivatives of anisotropic Matern covariance
 // [[Rcpp::export]]
-arma::cube d_matern_anisotropic2D(NumericVector covparms, NumericMatrix locs ){
+arma::cube d_matern_anisotropic2D(arma::vec covparms, arma::mat locs ){
 
     // covparms(0) = sigmasq
     // covparms(1) = L00
@@ -96,19 +95,19 @@ arma::cube d_matern_anisotropic2D(NumericVector covparms, NumericMatrix locs ){
     // nugget = sigmasq*tausq
     // overall variance = sigmasq*(1 + tausq) = sigmasq + nugget
 
-    int n = locs.nrow();
+    int n = locs.n_rows;
     //double nugget = covparms( 0 )*covparms( 5 );
-    double normcon = covparms(0)/(pow(2.0,covparms(4)-1.0)*Rf_gammafn(covparms(4)));
+    double normcon = covparms(0)/(pow(2.0,covparms(4)-1.0)*boost::math::tgamma(covparms(4) ));
     double eps = 1e-8;
     double normconeps = 
-        covparms(0)/(pow(2.0,covparms(4)+eps-1.0)*Rf_gammafn(covparms(4)+eps));
+        covparms(0)/(pow(2.0,covparms(4)+eps-1.0)*boost::math::tgamma(covparms(4)+eps ));
     
     double b0 = covparms(1)*covparms(1);
     double b1 = covparms(2)*covparms(2)+covparms(3)*covparms(3);
     double b2 = covparms(2)*covparms(3);
 
     // calculate derivatives
-    arma::cube dcovmat = arma::cube(n,n,covparms.length(), fill::zeros);
+    arma::cube dcovmat = arma::cube(n,n,covparms.n_elem, fill::zeros);
     for(int i1=0; i1<n; i1++){ for(int i2=0; i2<=i1; i2++){
         
         // calculate rescaled distance
@@ -122,26 +121,26 @@ arma::cube d_matern_anisotropic2D(NumericVector covparms, NumericMatrix locs ){
             cov = covparms(0);
             dcovmat(i1,i2,0) += 1.0;
         } else {
-            cov = normcon*pow( d, covparms(4) )*Rf_bessel_k(d,covparms(4),1.0);
+            cov = normcon*pow( d, covparms(4) )*boost::math::cyl_bessel_k(covparms(4), d);
             // variance parameter
             dcovmat(i1,i2,0) += cov/covparms(0);
             // cholesky parameters
             double cov_nu_m1 = normcon*pow(d,covparms(4)-1.0)*
-                Rf_bessel_k(d,covparms(4)-1.0,1.0);  
+                boost::math::cyl_bessel_k(covparms(4)-1.0, d);  
             dcovmat(i1,i2,1) -= cov_nu_m1*(h0*h0*covparms(1));
             dcovmat(i1,i2,2) -= cov_nu_m1*(h1*h1*covparms(2) + h0*h1*covparms(3));
             dcovmat(i1,i2,3) -= cov_nu_m1*(h1*h1*covparms(3) + h0*h1*covparms(2));
             
             // smoothness parameter (finite differencing)
             dcovmat(i1,i2,4) += 
-                ( normconeps*pow(d,covparms(4)+eps)*Rf_bessel_k(d,covparms(4)+eps,1.0) -
+                ( normconeps*pow(d,covparms(4)+eps)*boost::math::cyl_bessel_k(covparms(4) + eps, d) -
                   cov )/eps;
         }
         if( i1 == i2 ){ // update diagonal entry
             dcovmat(i1,i2,0) += covparms(5);
             dcovmat(i1,i2,5) += covparms(0); 
         } else { // fill in opposite entry
-            for(int j=0; j<covparms.length(); j++){
+            for(int j=0; j<covparms.n_elem; j++){
                 dcovmat(i2,i1,j) = dcovmat(i1,i2,j);
             }
         }
@@ -174,7 +173,7 @@ arma::cube d_matern_anisotropic2D(NumericVector covparms, NumericMatrix locs ){
 //' The nugget value \eqn{ \sigma^2 \tau^2 } is added to the diagonal of the covariance matrix.
 //' NOTE: the nugget is \eqn{ \sigma^2 \tau^2 }, not \eqn{ \tau^2 }. 
 // [[Rcpp::export]]
-arma::mat matern_anisotropic3D(NumericVector covparms, NumericMatrix locs ){
+arma::mat matern_anisotropic3D(arma::vec covparms, arma::mat locs ){
     
     // covparms(0) = sigmasq
     // covparms(1) = L00
@@ -188,11 +187,11 @@ arma::mat matern_anisotropic3D(NumericVector covparms, NumericMatrix locs ){
     // nugget = sigmasq*tausq
     // overall variance = sigmasq*(1 + tausq) = sigmasq + nugget
     
-    //int dim = locs.ncol();
-    int n = locs.nrow();
+    //int dim = locs.n_cols;
+    int n = locs.n_rows;
     double nugget = covparms( 0 )*covparms( 8 );
     double smooth = covparms( 7 );
-    double normcon = covparms(0)/(pow(2.0,smooth-1.0)*Rf_gammafn(smooth));
+    double normcon = covparms(0)/(pow(2.0,smooth-1.0)*boost::math::tgamma(smooth));
     
     // calculate covariances
     arma::mat covmat(n,n);
@@ -215,7 +214,7 @@ arma::mat matern_anisotropic3D(NumericVector covparms, NumericMatrix locs ){
             } else {
                 // calculate covariance            
                 covmat(i2,i1) = normcon*
-                    pow( d, smooth )*Rf_bessel_k(d,smooth,1.0);
+                    pow( d, smooth )*boost::math::cyl_bessel_k(smooth, d);
             }
             // add nugget
             if( i1 == i2 ){ covmat(i2,i2) += nugget; } 
@@ -228,7 +227,7 @@ arma::mat matern_anisotropic3D(NumericVector covparms, NumericMatrix locs ){
 
 //' @describeIn matern_anisotropic3D Derivatives of anisotropic Matern covariance
 // [[Rcpp::export]]
-arma::cube d_matern_anisotropic3D(NumericVector covparms, NumericMatrix locs ){
+arma::cube d_matern_anisotropic3D(arma::vec covparms, arma::mat locs ){
 
     // covparms(0) = sigmasq
     // covparms(1) = L00
@@ -242,16 +241,16 @@ arma::cube d_matern_anisotropic3D(NumericVector covparms, NumericMatrix locs ){
     // nugget = sigmasq*tausq
     // overall variance = sigmasq*(1 + tausq) = sigmasq + nugget
 
-    int n = locs.nrow();
+    int n = locs.n_rows;
     //double nugget = covparms( 0 )*covparms( 8 );
     double smooth = covparms( 7 );
-    double normcon = covparms(0)/(pow(2.0,smooth-1.0)*Rf_gammafn(smooth));
+    double normcon = covparms(0)/(pow(2.0,smooth-1.0)*boost::math::tgamma(smooth));
     double eps = 1e-8;
     double normconeps = 
-        covparms(0)/(pow(2.0,smooth+eps-1.0)*Rf_gammafn(smooth+eps));
+        covparms(0)/(pow(2.0,smooth+eps-1.0)*boost::math::tgamma(smooth+eps));
     
     // calculate derivatives
-    arma::cube dcovmat = arma::cube(n,n,covparms.length(), fill::zeros);
+    arma::cube dcovmat = arma::cube(n,n,covparms.n_elem, fill::zeros);
     for(int i2=0; i2<n; i2++){ for(int i1=0; i1<=i2; i1++){
         
         // calculate rescaled distance
@@ -270,12 +269,12 @@ arma::cube d_matern_anisotropic3D(NumericVector covparms, NumericMatrix locs ){
             cov = covparms(0);
             dcovmat(i1,i2,0) += 1.0;
         } else {
-            cov = normcon*pow( d, smooth )*Rf_bessel_k( d, smooth, 1.0 );
+            cov = normcon*pow( d, smooth )*boost::math::cyl_bessel_k(smooth, d);
             // variance parameter
             dcovmat(i1,i2,0) += cov/covparms(0);
             // cholesky parameters
             double cov_nu_m1 = normcon*pow( d, smooth - 1.0 )*
-                Rf_bessel_k( d, smooth - 1.0, 1.0 );  
+                boost::math::cyl_bessel_k(smooth - 1.0, d);  
             double Limhm = covparms(1)*h0;
                 dcovmat(i1,i2,1) = -cov_nu_m1*Limhm*h0;
             Limhm = covparms(2)*h0 + covparms(3)*h1;
@@ -288,14 +287,14 @@ arma::cube d_matern_anisotropic3D(NumericVector covparms, NumericMatrix locs ){
     
             // smoothness parameter (finite differencing)
             dcovmat(i1,i2,7) += 
-                ( normconeps*pow(d,smooth+eps)*Rf_bessel_k(d,smooth+eps,1.0) -
+                ( normconeps*pow(d,smooth+eps)*boost::math::cyl_bessel_k(smooth + eps, d) -
                   cov )/eps;
         }
         if( i1 == i2 ){ // update diagonal entry
             dcovmat(i1,i2,0) += covparms(8);
             dcovmat(i1,i2,8) += covparms(0); 
         } else { // fill in opposite entry
-            for(int j=0; j<covparms.length(); j++){
+            for(int j=0; j<covparms.n_elem; j++){
                 dcovmat(i2,i1,j) = dcovmat(i1,i2,j);
             }
         }
@@ -330,7 +329,7 @@ arma::cube d_matern_anisotropic3D(NumericVector covparms, NumericMatrix locs ){
 //' The nugget value \eqn{ \sigma^2 \tau^2 } is added to the diagonal of the covariance matrix.
 //' NOTE: the nugget is \eqn{ \sigma^2 \tau^2 }, not \eqn{ \tau^2 }. 
 // [[Rcpp::export]]
-arma::mat exponential_anisotropic2D(NumericVector covparms, NumericMatrix locs ){
+arma::mat exponential_anisotropic2D(arma::vec covparms, arma::mat locs ){
     
     // covparms(0) = sigmasq
     // covparms(1) = L00
@@ -340,8 +339,8 @@ arma::mat exponential_anisotropic2D(NumericVector covparms, NumericMatrix locs )
     // nugget = sigmasq*tausq
     // overall variance = sigmasq*(1 + tausq) = sigmasq + nugget
     
-    //int dim = locs.ncol();
-    int n = locs.nrow();
+    //int dim = locs.n_cols;
+    int n = locs.n_rows;
     double nugget = covparms( 0 )*covparms( 4 );
 
     // calculate covariances
@@ -375,7 +374,7 @@ arma::mat exponential_anisotropic2D(NumericVector covparms, NumericMatrix locs )
 
 //' @describeIn exponential_anisotropic2D Derivatives of anisotropic exponential covariance
 // [[Rcpp::export]]
-arma::cube d_exponential_anisotropic2D(NumericVector covparms, NumericMatrix locs ){
+arma::cube d_exponential_anisotropic2D(arma::vec covparms, arma::mat locs ){
 
     // covparms(0) = sigmasq
     // covparms(1) = L00
@@ -385,11 +384,11 @@ arma::cube d_exponential_anisotropic2D(NumericVector covparms, NumericMatrix loc
     // nugget = sigmasq*tausq
     // overall variance = sigmasq*(1 + tausq) = sigmasq + nugget
 
-    int n = locs.nrow();
+    int n = locs.n_rows;
     //double nugget = covparms( 0 )*covparms( 4 );
 
     // calculate derivatives
-    arma::cube dcovmat = arma::cube(n,n,covparms.length(), fill::zeros);
+    arma::cube dcovmat = arma::cube(n,n,covparms.n_elem, fill::zeros);
     for(int i2=0; i2<n; i2++){ for(int i1=0; i1<=i2; i1++){
         
         // calculate rescaled distance
@@ -421,7 +420,7 @@ arma::cube d_exponential_anisotropic2D(NumericVector covparms, NumericMatrix loc
             dcovmat(i1,i2,0) += covparms(4);
             dcovmat(i1,i2,4) += covparms(0); 
         } else { // fill in opposite entry
-            for(int j=0; j<covparms.length(); j++){
+            for(int j=0; j<covparms.n_elem; j++){
                 dcovmat(i2,i1,j) = dcovmat(i1,i2,j);
             }
         }
@@ -460,7 +459,7 @@ arma::cube d_exponential_anisotropic2D(NumericVector covparms, NumericMatrix loc
 //' The nugget value \eqn{ \sigma^2 \tau^2 } is added to the diagonal of the covariance matrix.
 //' NOTE: the nugget is \eqn{ \sigma^2 \tau^2 }, not \eqn{ \tau^2 }. 
 // [[Rcpp::export]]
-arma::mat exponential_anisotropic3D(NumericVector covparms, NumericMatrix locs ){
+arma::mat exponential_anisotropic3D(arma::vec covparms, arma::mat locs ){
     
     // covparms(0) = sigmasq
     // covparms(1) = L00
@@ -473,8 +472,8 @@ arma::mat exponential_anisotropic3D(NumericVector covparms, NumericMatrix locs )
     // nugget = sigmasq*tausq
     // overall variance = sigmasq*(1 + tausq) = sigmasq + nugget
     
-    //int dim = locs.ncol();
-    int n = locs.nrow();
+    //int dim = locs.n_cols;
+    int n = locs.n_rows;
     double nugget = covparms( 0 )*covparms( 7 );
 
     // calculate covariances
@@ -510,7 +509,7 @@ arma::mat exponential_anisotropic3D(NumericVector covparms, NumericMatrix locs )
 
 //' @describeIn exponential_anisotropic3D Derivatives of anisotropic exponential covariance
 // [[Rcpp::export]]
-arma::cube d_exponential_anisotropic3D(NumericVector covparms, NumericMatrix locs ){
+arma::cube d_exponential_anisotropic3D(arma::vec covparms, arma::mat locs ){
 
     // covparms(0) = sigmasq
     // covparms(1) = L00
@@ -523,11 +522,11 @@ arma::cube d_exponential_anisotropic3D(NumericVector covparms, NumericMatrix loc
     // nugget = sigmasq*tausq
     // overall variance = sigmasq*(1 + tausq) = sigmasq + nugget
 
-    int n = locs.nrow();
+    int n = locs.n_rows;
     double nugget = covparms( 0 )*covparms( 7 );
 
     // calculate derivatives
-    arma::cube dcovmat = arma::cube(n,n,covparms.length(), fill::zeros);
+    arma::cube dcovmat = arma::cube(n,n,covparms.n_elem, fill::zeros);
     for(int i2=0; i2<n; i2++){ for(int i1=0; i1<=i2; i1++){
         
         // calculate rescaled distance
@@ -565,7 +564,7 @@ arma::cube d_exponential_anisotropic3D(NumericVector covparms, NumericMatrix loc
             dcovmat(i1,i2,0) += covparms(7);
             dcovmat(i1,i2,7) += covparms(0); 
         } else { // fill in opposite entry
-            for(int j=0; j<covparms.length(); j++){
+            for(int j=0; j<covparms.n_elem; j++){
                 dcovmat(i2,i1,j) = dcovmat(i1,i2,j);
             }
         }
